@@ -5,7 +5,7 @@
 @Author: Kermit
 @Date: 2022-12-13 16:19:45
 @LastEditors: Kermit
-@LastEditTime: 2022-12-31 10:28:25
+@LastEditTime: 2023-01-09 23:22:23
 '''
 
 import traceback
@@ -16,7 +16,7 @@ import time
 import requests
 import json
 from .config_loader import ConfigLoader
-from .config import Algoinfo
+from .config import Algoinfo, get_service_status_url, start_build_url, reset_status_url, get_build_ws_url, start_deploy_url, get_deploy_ws_url, upload_code_url
 from algospace.login import login_instance, login
 from algospace.provider.enroll import enroll
 
@@ -78,8 +78,14 @@ def run_cloud_deploy(config_path: str, reset: bool = False):
             exit(1)
 
     if get_service_status(algorithm_config.name, algorithm_config.version) in ('built', 'building', 'pending', 'unready'):
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Build log:')
-        get_build_log(algorithm_config.name, algorithm_config.version)
+        try:
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Build log:')
+            get_build_log(algorithm_config.name, algorithm_config.version)
+        except Exception as e:
+            traceback.print_exc()
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                  '[AlgoSpace] Get build log error:', str(e))
+            exit(1)
 
         while get_service_status(algorithm_config.name, algorithm_config.version) in ('building', 'pending'):
             time.sleep(1)
@@ -107,12 +113,18 @@ def run_cloud_deploy(config_path: str, reset: bool = False):
             exit(1)
 
     if get_service_status(algorithm_config.name, algorithm_config.version) in ('deployed', 'deploying', 'waiting', 'built'):
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-              '[AlgoSpace] It may take a few minutes to distribute the image to calculation node.')
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-              '[AlgoSpace] And then the deploy log will be shown below. Please wait patiently.')
-        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Deploy log:')
-        get_deploy_log(algorithm_config.name, algorithm_config.version)
+        try:
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                  '[AlgoSpace] It may take a few minutes to distribute the image to calculation node.')
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                  '[AlgoSpace] And then the deploy log will be shown below. Please wait patiently.')
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Deploy log:')
+            get_deploy_log(algorithm_config.name, algorithm_config.version)
+        except Exception as e:
+            traceback.print_exc()
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                  '[AlgoSpace] Get deploy log error:', str(e))
+            exit(1)
 
         while get_service_status(algorithm_config.name, algorithm_config.version) in ('deploying', 'waiting'):
             time.sleep(1)
@@ -141,16 +153,20 @@ def show_running_log(config_path: str):
         exit(1)
 
     if get_service_status(algorithm_config.name, algorithm_config.version) == 'deployed':
-        get_deploy_log(algorithm_config.name, algorithm_config.version, True)
+        try:
+            get_deploy_log(algorithm_config.name, algorithm_config.version, True)
+        except Exception as e:
+            traceback.print_exc()
+            print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                  '[AlgoSpace] Get running log error:', str(e))
+            exit(1)
     else:
         print('[AlgoSpace] The service is not deployed. Please run `algospace cloud:deploy` first.')
 
 
 def get_service_status(name: str, version: str):
     ''' 获取服务状态 '''
-    algorithm_info = Algoinfo(name, version)
-
-    response = requests.get(algorithm_info.get_service_status_url,
+    response = requests.get(get_service_status_url,
                             params={'algoname': name, 'version': version},
                             headers=login_instance.get_header())
     if response.status_code != 200 and response.status_code != 201:
@@ -173,13 +189,15 @@ def upload_local_file_as_zip(name: str, version: str):
                         continue
                     file_path = os.path.join(root, file)
                     relative_path = file_path.replace(root_path, '')
-                    zip_file.write(file_path, relative_path)\
+                    zip_file.write(file_path, relative_path)
 
         algorithm_info = Algoinfo(name, version)
 
         with open(zip_name, 'rb') as f:
             files = {'file': (zip_name, f.read())}
-        response = requests.post(algorithm_info.upload_code_url, files=files, headers=login_instance.get_header())
+        response = requests.post(f'{upload_code_url}/{name}/{version}',
+                                 files=files,
+                                 headers=login_instance.get_header())
         if response.status_code != 200 and response.status_code != 201:
             raise Exception(response.status_code, response.content.decode())
         if response.json()['status'] != 200:
@@ -191,9 +209,7 @@ def upload_local_file_as_zip(name: str, version: str):
 
 def start_build_image(name: str, version: str, config_path: str):
     ''' 开始构建镜像 '''
-    algorithm_info = Algoinfo(name, version)
-
-    response = requests.post(algorithm_info.start_build_url, json={
+    response = requests.post(start_build_url, json={
         'algoname': name,
         'version': version,
         'config_path': config_path,  # TODO: 兼容 windows 路径格式
@@ -206,9 +222,7 @@ def start_build_image(name: str, version: str, config_path: str):
 
 def reset_status_image(name: str, version: str):
     ''' 重置算法状态 '''
-    algorithm_info = Algoinfo(name, version)
-
-    response = requests.post(algorithm_info.reset_status_url, json={
+    response = requests.post(reset_status_url, json={
         'algoname': name,
         'version': version,
     }, headers=login_instance.get_header())
@@ -220,8 +234,6 @@ def reset_status_image(name: str, version: str):
 
 def get_build_log(name: str, version: str, last_log_row_num: int = 0):
     ''' 获取构建日志 '''
-    algorithm_info = Algoinfo(name, version)
-
     def on_message(ws: websocket.WebSocket, message):
         nonlocal last_log_row_num
         result = json.loads(message)
@@ -229,9 +241,11 @@ def get_build_log(name: str, version: str, last_log_row_num: int = 0):
         print(add_prefix_to_log(result['log'], 'Cloud Build Log'))
 
     def on_close(ws: websocket.WebSocket, close_status_code: int, close_msg: str):
-        if close_status_code != 1000 and close_status_code < 4000:
+        if close_status_code and close_status_code != 1000 and close_status_code < 4000:
             # 非应用的异常码则重新连接
             get_build_log(name, version, last_log_row_num)
+        else:
+            print(f'Get log error: {close_status_code} {close_msg}')
 
     def on_open(ws: websocket.WebSocket):
         ws.send(json.dumps({
@@ -240,7 +254,7 @@ def get_build_log(name: str, version: str, last_log_row_num: int = 0):
             'last_log_row_num': last_log_row_num,
         }))
 
-    ws = websocket.WebSocketApp(algorithm_info.get_build_ws_url,
+    ws = websocket.WebSocketApp(get_build_ws_url,
                                 header=login_instance.get_header(),
                                 on_open=on_open,
                                 on_message=on_message,
@@ -250,9 +264,7 @@ def get_build_log(name: str, version: str, last_log_row_num: int = 0):
 
 def start_deploy(name: str, version: str):
     ''' 开始部署 '''
-    algorithm_info = Algoinfo(name, version)
-
-    response = requests.post(algorithm_info.start_deploy_url, json={
+    response = requests.post(start_deploy_url, json={
         'algoname': name,
         'version': version,
     }, headers=login_instance.get_header())
@@ -264,18 +276,22 @@ def start_deploy(name: str, version: str):
 
 def get_deploy_log(name: str, version: str, keep_after_success: bool = False, last_log_row_num: int = 0):
     ''' 获取部署日志 '''
-    algorithm_info = Algoinfo(name, version)
 
     def on_message(ws: websocket.WebSocket, message):
         nonlocal last_log_row_num
+        # websocket-client 限制消息体大小 64kb，需要注意
         result = json.loads(message)
+        if result['is_cover_previous']:
+            print(f'\033[{last_log_row_num}A\r', end='')
         last_log_row_num = result['last_log_row_num']
         print(add_prefix_to_log(result['log'], 'Cloud Deploy Log'))
 
     def on_close(ws: websocket.WebSocket, close_status_code: int, close_msg: str):
-        if close_status_code != 1000 and close_status_code < 4000:
+        if close_status_code and close_status_code != 1000 and close_status_code < 4000:
             # 非应用的异常码则重新连接
             get_deploy_log(name, version, keep_after_success, last_log_row_num)
+        else:
+            print(f'Get log error: {close_status_code} {close_msg}')
 
     def on_open(ws: websocket.WebSocket):
         ws.send(json.dumps({
@@ -285,7 +301,7 @@ def get_deploy_log(name: str, version: str, keep_after_success: bool = False, la
             'last_log_row_num': last_log_row_num,
         }))
 
-    ws = websocket.WebSocketApp(algorithm_info.get_deploy_ws_url,
+    ws = websocket.WebSocketApp(get_deploy_ws_url,
                                 header=login_instance.get_header(),
                                 on_open=on_open,
                                 on_message=on_message,
