@@ -5,7 +5,7 @@
 @Author: Kermit
 @Date: 2022-11-05 16:46:46
 @LastEditors: Kermit
-@LastEditTime: 2023-01-02 15:55:45
+@LastEditTime: 2023-01-10 21:43:46
 '''
 
 from typing import Callable, Optional, List, Tuple
@@ -22,6 +22,7 @@ import concurrent.futures
 import socket
 import traceback
 import requests
+import websocket
 from algospace.login import login, login_instance
 from .config_loader import ConfigLoader, valid_param_type
 from .enroll import enroll, verify_config, is_component_normal
@@ -63,7 +64,8 @@ class FnService:
                fn_req_queue_list: List[multiprocessing.Queue],
                fn_res_queue_list: List[multiprocessing.Queue]) -> None:
         ''' 启动函数服务 '''
-        self.algorithm_config.verify_service()  # 校验函数配置并附带将函数 import 入进程
+        # 校验函数配置并附带将函数 import 入进程
+        self.algorithm_config.verify_service()
         with concurrent.futures.ThreadPoolExecutor() as pool:
             pool.map(self.launch_thread,
                      [i for i in range(len(fn_req_queue_list))],
@@ -91,7 +93,7 @@ class ApiService:
 
     def read_file(self, path: str) -> str:
         ''' 将 storage 返回的 path 转换为本地 path '''
-        file_url = self.algorithm_info.storage_file_url + '/' + path
+        file_url = config.storage_file_url + '/' + path
         tmp_path = self.algorithm_config.service_tmp_path
         dir_path = os.path.dirname(os.path.join(tmp_path, path))
         if not os.path.exists(dir_path):
@@ -115,7 +117,7 @@ class ApiService:
         with open(local_path, 'rb') as f:
             files = {'file': (filename, f.read())}
 
-        upload_url = f'{self.algorithm_info.storage_file_url}/{self.algorithm_info.name}/{self.algorithm_info.version}'
+        upload_url = f'{config.storage_file_url}/{self.algorithm_info.name}/{self.algorithm_info.version}'
         response = requests.post(upload_url, files=files, headers=login_instance.get_header())
         if response.status_code != 200 and response.status_code != 201 and response.json()['status'] != 200:
             raise Exception('Failed to write file.')
@@ -211,6 +213,7 @@ class GradioService:
     def __init__(self, algorithm_config: ConfigLoader, algorithm_info: Algoinfo) -> None:
         self.algorithm_config = algorithm_config
         self.algorithm_info = algorithm_info
+        self.gradio_upload_url = None
 
     def get_type_component(self, type: str, describe: str):
         ''' 获取处理每种类型的 Gradio component '''
@@ -317,7 +320,8 @@ class GradioService:
 
     def launch_self(self, gradio_port_con: Connection) -> None:
         ''' 启动自主编写的 Gradio 服务 '''
-        self.algorithm_config.verify_gradio_launch()  # 校验自主实现的 Gradio 配置并附带将应用 import 入进程
+        # 校验自主实现的 Gradio 配置并附带将应用 import 入进程
+        self.algorithm_config.verify_gradio_launch()
         threading.Thread(target=self.check_launched, args=(gradio_port_con,), daemon=True).start()
         with GradioPrint():
             self.algorithm_config.gradio_launch_fn()  # 启动 Gradio 服务
@@ -379,7 +383,7 @@ class GradioService:
                 if os.path.isdir(filepath):
                     await upload_file(bashpath, os.path.join(subpath, filename))
                 else:
-                    url = self.algorithm_info.gradio_upload_url + '/' + subpath + '/' + filename
+                    url = self.gradio_upload_url + '/' + subpath + '/' + filename  # type: ignore
                     file = open(filepath, 'r', encoding="UTF-8", errors='ignore')
                     res = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(url, files={'file': file}, headers=login_instance.get_header()))
                     file.close()
@@ -393,7 +397,7 @@ class GradioService:
             html_file = open('./frontend/index.html', 'w+', encoding="UTF-8", errors='ignore')
             html_file.write(html_text)
             html_file.close()
-        url = self.algorithm_info.gradio_upload_url + '/index.html'
+        url = self.gradio_upload_url + '/index.html'  # type: ignore
         file = open('./frontend/index.html', 'r', encoding="UTF-8", errors='ignore')
         # 上传 index.html
         res = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(url, files={'file': file}, headers=login_instance.get_header()))
@@ -411,9 +415,11 @@ class Service:
     def __init__(
         self,
         config_path: str,
+        fetch_mode: str,
     ):
         self.algorithm_config = ConfigLoader(config_path)
         self.algorithm_info = Algoinfo(self.algorithm_config.name, self.algorithm_config.version)
+        self.fetch_mode = fetch_mode
 
         self.login_headers = {}
         self.login_headers_timestamp = 0
@@ -466,49 +472,134 @@ class Service:
         self.algorithm_config.gradio_server_port = gradio_port
 
         # 开始处理
+        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}] [Service] Running in {self.fetch_mode} mode.')
         print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}] [Service] Your algorithm site: {self.algorithm_info.algorithm_site}')
         print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}] [Service] Start handling...')
         print('')
 
-        async def start():
-            parallel = {
-                'max': self.algorithm_config.service_max_parallel,
-                'curr': 0,
-            }
-            loop = asyncio.get_event_loop()
-            while True:
-                while parallel['curr'] >= parallel['max']:
+        parallel = {
+            'max': self.algorithm_config.service_max_parallel,
+            'curr': 0,
+        }
+
+        def is_parallel_full():
+            return parallel['curr'] >= parallel['max']
+
+        def add_parallel(is_full: bool = False):
+            parallel['curr'] += 1 if not is_full else parallel['max']
+
+        def minus_parallel(is_full: bool = False):
+            parallel['curr'] -= 1 if not is_full else parallel['max']
+
+        def get_rest_parallel():
+            return parallel['max'] - parallel['curr']
+
+        if self.fetch_mode == 'listen':
+            def on_init():
+                while is_parallel_full():
                     # 并行数达到上限，等待
-                    await asyncio.sleep(config.wait_interval)
-
-                # 请求消息
-                try:
-                    req_info = await self.ask_data()
-                    if req_info is None:
-                        await asyncio.sleep(config.wait_interval)
-                        continue
-                except Exception as e:
-                    traceback.print_exc()
-                    print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
-                          '[Service] Ask data error:', str(e))
-                    await asyncio.sleep(config.wait_interval)
-                    continue
+                    time.sleep(config.wait_interval)
+                return get_rest_parallel()
 
+            def on_req(req_info: dict, loop: asyncio.AbstractEventLoop):
                 # 处理请求
                 try:
-                    loop.create_task(self.handle(req_info, fn_lock_list,
-                                     fn_req_queue_list, fn_res_queue_list, parallel))
-                    await asyncio.sleep(config.call_interval)
+                    add_parallel(req_info['type'] == 'gradio_init')
+                    future = asyncio.run_coroutine_threadsafe(self.handle(req_info, fn_lock_list,
+                                                                          fn_req_queue_list, fn_res_queue_list), loop)
+                    future.add_done_callback(lambda task: minus_parallel(
+                        req_info['type'] == 'gradio_init'))  # type: ignore
                 except Exception as e:
                     traceback.print_exc()
                     print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
                           '[Service] Create handle task error:', str(e))
-                    await asyncio.sleep(config.wait_interval)
 
-        asyncio.run(start())
+                while is_parallel_full():
+                    # 并行数达到上限，等待
+                    time.sleep(config.wait_interval)
+                return get_rest_parallel()
 
-    async def ask_data(self):
-        ask_for_data_resp = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.get(self.algorithm_info.ask_data_url, headers=login_instance.get_header()))
+            asyncio.run(self.ws_ask_data(on_init, on_req))
+
+        elif self.fetch_mode == 'poll':
+            async def start():
+                loop = asyncio.get_event_loop()
+                while True:
+                    while is_parallel_full():
+                        # 并行数达到上限，等待
+                        await asyncio.sleep(config.wait_interval)
+
+                    # 请求消息
+                    try:
+                        req_info = await self.ask_data()
+                        if req_info is None:
+                            await asyncio.sleep(config.wait_interval)
+                            continue
+                    except Exception as e:
+                        traceback.print_exc()
+                        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                              '[Service] Ask data error:', str(e))
+                        await asyncio.sleep(config.wait_interval)
+                        continue
+
+                    # 处理请求
+                    try:
+                        add_parallel(req_info['type'] == 'gradio_init')
+                        future = loop.create_task(self.handle(req_info, fn_lock_list,
+                                                              fn_req_queue_list, fn_res_queue_list))
+                        future.add_done_callback(lambda task: minus_parallel(
+                            req_info['type'] == 'gradio_init'))  # type: ignore
+                        await asyncio.sleep(config.call_interval)
+                    except Exception as e:
+                        traceback.print_exc()
+                        print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
+                              '[Service] Create handle task error:', str(e))
+                        await asyncio.sleep(config.wait_interval)
+
+            asyncio.run(start())
+
+    async def ws_ask_data(self, on_init: Callable, on_req: Callable):
+        loop = asyncio.get_event_loop()
+
+        def on_message(ws: websocket.WebSocket, message):
+            # websocket-client 限制消息体大小 64kb，需要注意
+            result: dict = json.loads(message)
+            if result.get('ack'):
+                rest_parallel = on_init()
+                ws.send(json.dumps({
+                    'rest_parallel': rest_parallel,
+                }))
+            else:
+                req_info = result['req_info']
+                rest_parallel = on_req(req_info, loop)
+                ws.send(json.dumps({
+                    'rest_parallel': rest_parallel,
+                }))
+
+        def on_open(ws: websocket.WebSocket):
+            ws.send(json.dumps({
+                'algorithm_name': self.algorithm_info.name,
+                'algorithm_version': self.algorithm_info.version,
+            }))
+
+        ws = websocket.WebSocketApp(self.ws_ask_data_url,
+                                    header=login_instance.get_header(),
+                                    on_open=on_open,
+                                    on_message=on_message)
+
+        await loop.run_in_executor(None, ws.run_forever)
+
+        await asyncio.sleep(config.wait_interval)
+        await self.ws_ask_data(on_init, on_req)
+
+    async def ask_data(self, id: Optional[str] = None) -> Optional[dict]:
+        ask_for_data_resp = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.get(self.ask_data_url,
+                                                                                                      params={
+                                                                                                          'algorithm_name': self.algorithm_info.name,
+                                                                                                          'algorithm_version': self.algorithm_info.version,
+                                                                                                          **({'id': id} if id else {}),
+                                                                                                      },
+                                                                                                      headers=login_instance.get_header()))
         if ask_for_data_resp.status_code != 200 and ask_for_data_resp.status_code != 201:
             raise Exception(ask_for_data_resp.status_code, ask_for_data_resp.content.decode())
         ask_for_data_dict = ask_for_data_resp.json()
@@ -517,10 +608,10 @@ class Service:
             return
         if ask_for_data_dict['status'] != 200:
             raise Exception('the response', ask_for_data_dict)
-        req_info = ask_for_data_dict.get('data', {})
-        if type(req_info) != dict:
+        req_data = ask_for_data_dict.get('data', {})
+        if type(req_data) != dict:
             raise Exception('the property of \'data\' is not dict.')
-        req_info = req_info.get('req_info', {})
+        req_info = req_data.get('req_info', {})
         if type(req_info) != dict:
             raise Exception('the property of \'req_info\' is not dict.')
         return req_info
@@ -529,11 +620,26 @@ class Service:
                      req_info: dict,
                      fn_lock_list: List[Lock],
                      fn_req_queue_list: List[multiprocessing.Queue],
-                     fn_res_queue_list: List[multiprocessing.Queue],
-                     parallel: dict):
+                     fn_res_queue_list: List[multiprocessing.Queue]):
         try:
-            parallel['curr'] += 1
             try:
+                if type(req_info['data']) != dict:
+                    req_id = req_info['id']
+                    full_req_info = None
+                    retry_times = 0
+                    max_retry_times = 5
+                    while not full_req_info:
+                        if retry_times >= max_retry_times:
+                            raise Exception(f'Algo req \'{req_id}\' is not found.')
+                        try:
+                            full_req_info = await self.ask_data(req_id)
+                        except:
+                            pass
+                        finally:
+                            retry_times += 1
+                            await asyncio.sleep(config.wait_interval * retry_times)
+                    req_info = full_req_info
+
                 # 根据请求类型处理
                 if req_info['type'] == 'api':
                     # 处理计算请求
@@ -543,26 +649,29 @@ class Service:
                     result = await self.gradio_service.handle(req_info['data'])
                 elif req_info['type'] == 'gradio_init':
                     # 处理 Gradio 初始化请求
-                    try:
-                        parallel['curr'] += parallel['max']  # 初始化时要阻塞所有并行请求
-                        result = await self.gradio_service.handle_init()
-                    finally:
-                        parallel['curr'] -= parallel['max']
+                    result = await self.gradio_service.handle_init()
                 else:
                     result = {}
 
                 # 回传数据
                 res_info = {
                     'id': req_info['id'],
+                    'name': req_info['name'],
+                    'version': req_info['version'],
                     'type': req_info['type'],
-                    'status': 'finished',
+                    'status': 'success',
                     'owner': req_info['owner'],
                     'create_date': req_info['create_date'],
                     'result': result
                 }
-                return_ans_param = {'res_info': res_info}
-                return_ans_resp = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(self.algorithm_info.return_ans_url,
-                                                                                                             json=return_ans_param, headers=login_instance.get_header()))
+                return_ans_param = {
+                    'res_info': res_info,
+                    'algorithm_name': req_info['name'],
+                    'algorithm_version': req_info['version']
+                }
+                return_ans_resp = await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(self.return_ans_url,
+                                                                                                             json=return_ans_param,
+                                                                                                             headers=login_instance.get_header()))
 
                 if return_ans_resp.status_code != 200 and return_ans_resp.status_code != 201 and return_ans_resp.json()['status'] != 200:
                     err_msg = "Service Result Return Error."
@@ -575,22 +684,27 @@ class Service:
                 # 回传失败数据
                 res_info = {
                     'id': req_info['id'],
+                    'name': req_info['name'],
+                    'version': req_info['version'],
                     'type': req_info['type'],
                     'status': 'error',
                     'owner': req_info['owner'],
                     'create_date': req_info['create_date'],
                     'result': {'err_msg': str(e)}
                 }
-                return_error_param = {'res_info': res_info}
-                await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(self.algorithm_info.return_err_url, json=return_error_param,
+                return_error_param = {
+                    'res_info': res_info,
+                    'algorithm_name': req_info['name'],
+                    'algorithm_version': req_info['version']
+                }
+                await asyncio.get_event_loop().run_in_executor(None, lambda: requests.post(self.return_err_url,
+                                                                                           json=return_error_param,
                                                                                            headers=login_instance.get_header()))
                 print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
                       '[Service] Return error ans success.')
         except Exception as e:
             traceback.print_exc()
             print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[Service] Handle error:', str(e))
-        finally:
-            parallel['curr'] -= 1
 
     def login(self):
         if not login(self.algorithm_config.username, self.algorithm_config.password):
@@ -638,7 +752,13 @@ class Service:
 
     def is_component_normal(self):
         while (True):
-            if is_component_normal(self.algorithm_config.name, self.algorithm_config.version):
+            res = is_component_normal(self.algorithm_config.name, self.algorithm_config.version)
+            if res['is_component_normal']:
+                self.ask_data_url = res['ask_data_url']
+                self.ws_ask_data_url = res['ws_ask_data_url']
+                self.return_ans_url = res['return_ans_url']
+                self.return_err_url = res['return_err_url']
+                self.gradio_service.gradio_upload_url = res['gradio_upload_url']
                 break
             print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
                   f'[{self.algorithm_info.upper_name}] Waiting for components started...')
@@ -649,7 +769,7 @@ class Service:
             'algorithm_name': self.algorithm_info.name,
             'algorithm_version': self.algorithm_info.version
         }
-        requests.post(self.algorithm_info.heartbeat_url, data=body, headers=login_instance.get_header())
+        requests.post(config.heartbeat_url, data=body, headers=login_instance.get_header())
 
     async def start(self):
         # 初始化状态
@@ -751,15 +871,22 @@ class Service:
                           f'[{self.algorithm_info.upper_name}] Heartbeat error:', str(e))
 
         async def subprocess_stdio_task():
-            stdio_exec = QueueStdIOExec(stdio_queue).exec
+            queue_stdio_exec = QueueStdIOExec(stdio_queue)
+            stdio_exec = queue_stdio_exec.exec
+            stdio_exec_all = queue_stdio_exec.exec_all
+            is_execed = True
             while True:
                 try:
-                    await asyncio.sleep(0.1)
-                    await asyncio.get_running_loop().run_in_executor(None, stdio_exec)
+                    if not is_execed:
+                        await asyncio.sleep(0.1)
+                    is_execed = await asyncio.get_running_loop().run_in_executor(None, stdio_exec)
                 except Exception as e:
                     traceback.print_exc()
                     print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]',
                           f'[{self.algorithm_info.upper_name}] Handle subprocess stdio error:', str(e))
+                except asyncio.CancelledError:
+                    stdio_exec_all()
+                    raise
 
         # 当有任务抛出异常时，停止所有任务
         tasks = [alive_task(), heartbeat_task(), subprocess_stdio_task()]
@@ -770,11 +897,11 @@ class Service:
             task.result()
 
 
-def run_service(config_path: str) -> None:
+def run_service(config_path: str, fetch_mode: str = 'listen') -> None:
     loop = asyncio.get_event_loop()
     try:
         print(f'[{time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())}]', '[AlgoSpace] Init.')
-        service = Service(config_path)
+        service = Service(config_path, fetch_mode)
         loop.run_until_complete(service.start())
     except:
         traceback.print_exc()
